package ppJava.serie04.oceanlife;

import java.util.Random;

/**
 * A fish, swimming in the ocean
 * 
 * @author ken, bpei
 * 
 */
public class Fish extends OceanObject {

	private static final long serialVersionUID = 1L;

	int randomSprite = new Random().nextInt(3);
	int imageSizes[] = { 50, 100, 100, 75 };

	String imageNormalResource = "res/images/fish/fishSprite"
			+ String.valueOf(randomSprite) + ".png";
	String imageReversedResource = "res/images/fish/fishSprite"
			+ String.valueOf(randomSprite) + "_rev.png";

	// border where a fish changes its direction
	int paddingBorder = 100;

	/*
	 * (non-Javadoc)
	 * 
	 * @see ppJava.serie04.oceanlife.OceanObject#toString()
	 */
	@Override
	public String toString() {
		return "Fish" + super.toString() + "\n";
	}

	/**
	 * Constructor for a fish at position (x,y)
	 * 
	 * @param x
	 * @param y
	 */
	public Fish(int x, int y) {
		int pos[] = { x, y };
		this.setPosition(pos);
		this.setImageResource(imageNormalResource);
		this.setImageIcon(createImageIcon(this.getImageResource(), ""));
		int offset[] = { getImageIcon().getIconWidth() / 2,
				getImageIcon().getIconHeight() / 2 };
		this.setOffset(offset);
		this.setMoveVector(genRandomVector());
		this.setImageSize(imageSizes[randomSprite]);
		flipImage();
	}

	/**
	 * Turn around the image
	 */
	public void flipImage() {
		setImageResource((getMoveVector()[0] < 0) ? imageNormalResource
				: imageReversedResource);
		this.setImageIcon(createImageIcon(this.getImageResource(), ""));
	}

	/**
	 * Change the directions and turn around the image
	 */
	@Override
	public void changeDirections(OceanObject tempRefObject) {
		super.changeDirections(tempRefObject);
		flipImage();
	}

	/**
	 * The movement of a fish
	 */
	public void run() {

		int moveVector[] = getMoveVector();
		int signs[] = { moveVector[0], moveVector[1] };
		boolean changed = false;

		if (getPosition()[0] < paddingBorder) {
			signs[0] = 1;
			changed = true;
		}

		if (getPosition()[0] > getBorders()[0] - paddingBorder) {
			signs[0] = -1;
			changed = true;
		}

		if (getPosition()[1] < paddingBorder) {
			signs[1] = 1;
			changed = true;
		}

		if (getPosition()[1] > getBorders()[1] - paddingBorder) {
			signs[1] = -1;
			changed = true;
		}

		if (changed) {
			this.switchMovementDirection(signs);
			flipImage();
		}

		moveStep();
		
		this.checkBarrier();

	}
}
