package ppJava.serie04.oceanlife;

import java.io.IOException;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Ocean: Management of a LinkedList of OceanObjects
 * 
 * @see OceanInterface
 */
public class Ocean implements OceanInterface {

	private static Logger logger = Logger.getLogger("ppJava.serie04.oceanlife");

	private int width = 0;
	private int depth = 0;
	private LinkedList<OceanObject> oceanObjects = null;

	/**
	 * Constructor for an ocean with bounds and a already defined linked list of
	 * oceanObjects which can be null
	 * 
	 * @param width
	 * @param depth
	 * @param oceanObjects
	 *            - a list of OceanObjects
	 * @throws OceanException
	 *             - i.e. invalid list
	 */
	public Ocean(int width, int depth, LinkedList<OceanObject> oceanObjects)
			throws OceanException {
		this.setWidth(width);
		this.setDepth(depth);
		this.setOceanObjects(oceanObjects);
	}

	/**
	 * @return true if ocean is empty
	 */
	public boolean emptyOcean() {
		return (this.oceanObjects.size() == 0);
	}

	@Override
	public LinkedList<OceanObject> getOceanObjects() {
		// TODO Auto-generated method stub
		return this.oceanObjects;
	}

	@Override
	public int getDepth() {
		// TODO Auto-generated method stub
		return this.depth;
	}

	@Override
	public int getWidth() {
		// TODO Auto-generated method stub
		return this.width;
	}

	@Override
	public void setDepth(int depth) {
		// TODO Auto-generated method stub
		if (depth < 0)
			throw new IndexOutOfBoundsException();
		this.depth = depth;
	}

	@Override
	public void setWidth(int width) throws OceanException {
		// TODO Auto-generated method stub
		if (width < 0)
			throw new OceanException(
					"Invalid value for \"width\" in Ocean.setWidth(width)");
		this.width = width;
	}

	/**
	 * clears the linked list of oceanObjects
	 * 
	 * @see LinkedList
	 */
	public void clearObjectList() {
		this.getOceanObjects().clear();
	}

	@Override
	public void setOceanObjects(LinkedList<OceanObject> oceanObjects) {
		// TODO Auto-generated method stub

		if (oceanObjects != null)
			for (int i = 0; i < oceanObjects.size(); i++) {
				try {
					oceanObjects.get(i).checkPosition(this);
				} catch (OceanException e) {
					// TODO Auto-generated catch block
					logger.log(Level.INFO,
							"Object has invalid position and will be ignored: "
									+ oceanObjects.get(i).toString());
					// oceanObjects.remove(i);
				}
			}
		this.oceanObjects = oceanObjects;
	}

	@Override
	public void addOceanObject(OceanObject oceanObject) throws OceanException {

		int borders[] = { width, depth };

		oceanObject.setBorders(borders);
		oceanObject.checkPosition(this);
		oceanObjects.add(oceanObject);
		// new Thread(oceanObject).start();
	}

	public int calcAbsDistance(int pos1[], int pos2[]) {
		return (int) Math.sqrt((Math.pow(Math.abs(pos1[0] - pos2[0]), 2) + Math
				.pow(Math.abs(pos1[1] - pos2[1]), 2)));
	}

	@Override
	public int indexOfNearestObject(int[] position) throws OceanException {

		int nextObjectIndex = 0;
		int relativePos[] = { 0, 0 };

		if (!emptyOcean()) {
			int tempDiffPos[] = { this.getWidth(), this.getDepth() };

			int controlDistance = calcAbsDistance(relativePos, tempDiffPos);

			for (int i = 0; i < oceanObjects.size(); i++) {

				OceanObject tempObject = oceanObjects.get(i);
				int distance = calcAbsDistance(tempObject.getPosition(),
						position);

				if (distance > 0) {
					if (distance <= controlDistance) {
						nextObjectIndex = i;
						controlDistance = distance;
					}
				}
			}
		} else {
			throw new OceanException("Ocean is empty");
		}
		return nextObjectIndex;
	}

	@Override
	public void removeOceanObject(int index) throws IllegalArgumentException {

		if (index >= 0 && index < oceanObjects.size()) {
			oceanObjects.remove(index);
		} else {
			throw new IllegalArgumentException("Index not found");
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ppJava.serie04.oceanlife.OceanInterface#loadOcean(java.lang.String)
	 */
	@Override
	public OceanInterface loadOcean(String filename)
			throws ClassNotFoundException, IOException {
		return OceanFileOps.loadOceanObjectFile(filename, this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ppJava.serie04.oceanlife.OceanInterface#saveOcean(java.lang.String)
	 */
	@Override
	public void saveOcean(String filename) throws IOException {
		OceanFileOps.saveOceanObjectFile(filename, this);
	}

	@Override
	public String toString() {

		String result = "Ocean: \tdepth=" + depth + ", width=" + width + "\n"
				+ "\tCount:" + OceanObject.getObjectCount() + "\n"
				+ "\tObjects:" + "\n";

		for (int i = 0; i < oceanObjects.size(); i++) {
			result += "\t\t [" + String.valueOf(i) + "] "
					+ oceanObjects.get(i).toString();
		}

		return result;
	}

}
