package ppJava.serie04.oceanlife;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JPanel;

/**
 * A JPanel for drawing the ocean. Repainting is triggered from GUI
 * 
 * @see OceanGUI
 */
public class OceanCanvasPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private Image background;
	private Ocean ocean = null;

	/**
	 * Constructor for Panel. Loads the background-image.
	 */
	public OceanCanvasPanel() {
		setBorder(BorderFactory.createLineBorder(Color.black));
		background = createImageIcon("res/images/background2.png",
				"something like a aquarium").getImage();
	}

	/**
	 * Draws all oceanObjects of ocean. Middle of sprite is moved to objects
	 * position
	 * 
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);

		g.drawImage(background, 0, 0, this.getWidth(), this.getHeight(), null);

		for (OceanObject oceanObject : ocean.getOceanObjects()) {

			g.drawImage(oceanObject.getImageIcon().getImage(), oceanObject
					.getPosition()[0]
					- oceanObject.getOffset()[0], oceanObject.getPosition()[1]
					- oceanObject.getOffset()[1], null);

		}

	}

	/**
	 * This panel draws all items of this ocean
	 * 
	 * @param ocean
	 */
	public void setOcean(Ocean ocean) {
		this.ocean = ocean;
	}

	/**
	 * Returns an ImageIcon, or null if the path was invalid. (source: oracle)
	 * */
	protected ImageIcon createImageIcon(String path, String description) {
		java.net.URL imgURL = getClass().getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL, description);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}
}
