package ppJava.serie04.oceanlife;

import java.io.IOException;
import java.util.LinkedList;

/**
 * OceanInterface - Interface for simulating and manipulating an Ocean
 * 
 * - get/set for bounds of Ocean - get/set for linkedList of OceanObjects -
 * append/remove for linkedList of OceanObjects - compute the next item at a
 * specific position - method for a readable output of Ocean and OceanObjects
 */
public interface OceanInterface {

	/**
	 * Returns depth of ocean in pixels
	 * 
	 * @return width of ocean in pixels
	 */
	public int getWidth();

	/**
	 * Set the width of ocean
	 * 
	 * @param width
	 * @throws OceanException
	 *             - if width is invalid
	 */
	public void setWidth(int width) throws OceanException;

	/**
	 * Returns depth of ocean in pixels
	 * 
	 * @return depth of ocean in pixels
	 */
	public int getDepth();

	/**
	 * Set the depth of ocean
	 * 
	 * @param depth
	 * @throws OceanException
	 *             - if depth is invalid
	 */
	public void setDepth(int depth) throws OceanException;

	/**
	 * Returns a linked list of all OceanObjects
	 * 
	 * @return a linked list of all OceanObjects (null if empty)
	 */
	public LinkedList<OceanObject> getOceanObjects();

	/**
	 * Sets a list of OceanObjects
	 * 
	 * @param oceanObjects
	 */
	public void setOceanObjects(LinkedList<OceanObject> oceanObjects);

	/**
	 * Appends an oceanObject to the list
	 * 
	 * @param oceanObject
	 * @throws OceanException
	 *             - if data of oceanObject is invalid (i.e. position)
	 */
	public void addOceanObject(OceanObject oceanObject) throws OceanException;

	/**
	 * Removes an oceanObject at a specific position (index)
	 * 
	 * @param index
	 * @throws OceanException
	 *             - if index is invalid (i.e. out of bounds)
	 */
	public void removeOceanObject(int index) throws IllegalArgumentException;

	/**
	 * Compute the next item to a specific position
	 * 
	 * @param position
	 * @return the id of next oceanObject to the given position
	 * @throws OceanException
	 */
	public int indexOfNearestObject(int[] position) throws OceanException;

	/**
	 * Serialize and write all oceanObjects to a file
	 * 
	 * @param filename
	 * @throws IOException
	 */
	public void saveOcean(String filename) throws IOException;

	/**
	 * Reads and deserialize an objectStream from file
	 * 
	 * @param filename
	 * @return ocean with objectlist
	 * @throws ClassNotFoundException
	 * @throws IOException
	 */
	public OceanInterface loadOcean(String filename)
			throws ClassNotFoundException, IOException;

	/**
	 * Returns a readable output
	 * 
	 * @return a readable output of the ocean and its list
	 */
	public String toString();

}
