package ppJava.serie04.oceanlife;

import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

/**
 * A JMenuBar for file-operations (new/load/save) and quit. Event-messages are
 * defined in this class
 * 
 */
public class OceanMenuBar extends JMenuBar {

	private static final long serialVersionUID = 1L;

	/**
	 * A MenuItem consists of: Message, Label and Shortcut
	 */
	private class MenuItem {

		String menuAction;
		String menuLabel;
		int menuShortcut;

		/**
		 * Constructor
		 * 
		 * @param menuAction
		 * @param menuLabel
		 * @param menuShortcut
		 */
		public MenuItem(String menuAction, String menuLabel, int menuShortcut) {
			this.menuAction = menuAction;
			this.menuLabel = menuLabel;
			this.menuShortcut = menuShortcut;
		}
	}

	ArrayList<MenuItem> menuItemsFile = new ArrayList<MenuItem>();
	ArrayList<MenuItem> menuItemsSettings = new ArrayList<MenuItem>();
	ActionListener actionListener;

	/**
	 * Called by "populateMenu". Will populate a submenu with given list of
	 * menuItems
	 * 
	 * @param menuItems
	 *            - all items which should be added
	 * @param subMenuLabel
	 *            - a label for this submenu
	 * @param shortcut
	 * @param tooltip
	 */
	private void populateSubMenu(ArrayList<MenuItem> menuItems,
			String subMenuLabel, int shortcut, String tooltip) {

		JMenuItem tempItem;
		JMenu menu;

		menu = new JMenu(subMenuLabel);
		menu.setMnemonic(shortcut);
		menu.getAccessibleContext().setAccessibleDescription(tooltip);
		add(menu);

		for (MenuItem menuItem : menuItems) {
			tempItem = new JMenuItem(menuItem.menuLabel, menuItem.menuShortcut);
			tempItem.setActionCommand(menuItem.menuAction);
			tempItem.addActionListener(actionListener);
			menu.add(tempItem);
		}
	}

	/**
	 * Populates the menuBar. (Triggered from outside - @see OceanGUI)
	 */
	public void populateMenu() {

		menuItemsFile.add(new MenuItem("newOceanFile", "New", KeyEvent.VK_N));
		menuItemsFile.add(new MenuItem("loadOceanFile", "Load", KeyEvent.VK_L));
		menuItemsFile.add(new MenuItem("saveOceanFile", "Save", KeyEvent.VK_S));
		menuItemsFile.add(new MenuItem("quitGUI", "Quit", KeyEvent.VK_Q));

		populateSubMenu(menuItemsFile, "File", KeyEvent.VK_F,
				"some functionality");

		populateSubMenu(menuItemsSettings, "Settings", KeyEvent.VK_S,
				"more functionality");
	}

	public OceanMenuBar() {
		// Create the menu bar.
		super();
	}

	/**
	 * This ActionListener is set by OceanGUI
	 * 
	 * @see OceanGUI
	 * @param al
	 */
	public void setActionListener(ActionListener al) {
		// TODO Auto-generated method stub
		actionListener = al;
	}

}
