package ppJava.serie04.oceanlife;

import java.awt.Dimension;

import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

/**
 * A JToolBar for oceanObject-operations (create/remove) and control of
 * oceanlife. Event-messages are defined in this class
 */
public class OceanToolbar extends JToolBar {

	private static final long serialVersionUID = 1L;

	ActionListener actionListener;
	ArrayList<ToolBarItem> toolBarCreateObjectItems = new ArrayList<ToolBarItem>();
	ArrayList<ToolBarItem> toolBarSelectedActions = new ArrayList<ToolBarItem>();
	ArrayList<ToolBarItem> toolBarRuntimeActions = new ArrayList<ToolBarItem>();

	JComboBox toolComboBox;

	/**
	 * A ToolBarItem consists of: Message, Label and Icon
	 */
	private class ToolBarItem {

		String toolAction;
		@SuppressWarnings("unused")
		String toolLabel;
		String toolIconImage;

		/**
		 * Constructor
		 * 
		 * @param toolAction
		 * @param toolLabel
		 * @param toolIconImage
		 */
		public ToolBarItem(String toolAction, String toolLabel,
				String toolIconImage) {

			this.toolAction = toolAction;
			this.toolLabel = toolLabel;
			this.toolIconImage = toolIconImage;
		}
	}

	/**
	 * Called by "populateToolbar". Will populate a section with given list of
	 * sectionItems
	 * 
	 * @param sectionItems
	 */
	private void populateToolbarSection(ArrayList<ToolBarItem> sectionItems) {
		JToggleButton button;
		Icon icon;

		for (ToolBarItem toolItem : sectionItems) {
			icon = createImageIcon(toolItem.toolIconImage, "");
			button = new JToggleButton(icon);
			button.setActionCommand(toolItem.toolAction);
			button.addActionListener(actionListener);
			add(button);
		}
	}

	/**
	 * Populates the toolBar. (Triggered from outside - @see OceanGUI)
	 * 
	 */
	public void populateToolbar() {

		toolBarCreateObjectItems.add(new ToolBarItem("createOceanRock", "Rock",
				"res/images/menu/rockButtonSmall.png"));
		toolBarCreateObjectItems.add(new ToolBarItem("createOceanPlant",
				"Plant", "res/images/menu/plantButtonSmall.png"));
		toolBarCreateObjectItems.add(new ToolBarItem("createOceanFish", "Fish",
				"res/images/menu/fishButtonSmall.png"));
		toolBarCreateObjectItems.add(new ToolBarItem("createOceanBubble",
				"Bubble", "res/images/menu/bubbleButtonSmall.png"));

		populateToolbarSection(toolBarCreateObjectItems);

		addSeparator();
		toolComboBox = new JComboBox();
		toolComboBox.setMaximumSize(new Dimension(200, 32));
		toolComboBox.setToolTipText("OceanObjects");
		toolComboBox.addActionListener(actionListener);
		add(toolComboBox);

		addSeparator();
		toolBarSelectedActions.add(new ToolBarItem("removeOceanObject",
				"Index", "res/images/menu/ctrlKillButton.png"));

		populateToolbarSection(toolBarSelectedActions);

		addSeparator();
		toolBarRuntimeActions.add(new ToolBarItem("stopMovement", "Control",
				"res/images/menu/ctrlPauseButton.png"));
		toolBarRuntimeActions.add(new ToolBarItem("startMovement", "Control",
				"res/images/menu/ctrlPlayButton.png"));
		toolBarRuntimeActions.add(new ToolBarItem("traceMovement", "Control",
				"res/images/menu/ctrlStepButton.png"));

		populateToolbarSection(toolBarRuntimeActions);

	}

	/** Returns an ImageIcon, or null if the path was invalid. */
	protected ImageIcon createImageIcon(String path, String description) {
		java.net.URL imgURL = getClass().getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL, description);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}

	/**
	 * @return the toolComboBox
	 */
	public JComboBox getToolComboBox() {
		return toolComboBox;
	}

	/**
	 * @param toolComboBox
	 *            the toolComboBox to set
	 */
	public void setToolComboBox(JComboBox toolComboBox) {
		this.toolComboBox = toolComboBox;
	}

	/**
	 * This ActionListener is set by OceanGUI
	 * 
	 * @see OceanGUI
	 * @param al
	 */
	public void setActionListener(ActionListener al) {
		// TODO Auto-generated method stub
		actionListener = al;
	}

}
